page   64,120
; /************************************************************************
; *                                                                       *
; *   File Name : C0S.ASM                                                 *
; *   Purpose :   Initial execution code and assembly language functions  *
; *               for Hostess i Toolkit Control Program, Turbo C++        *
; *               version.                                                *
; *   Date:       February 5, 1991                                        *
; *   Author:     Howard D. Straus, Computer Software Systems, Carmel, CA *
; *   Release:    1.00 - Original release                                 *
;**************************************************************************
; *                         COPYRIGHT NOTICE                              *
; *=======================================================================*
; *   Copyright 1991 by Howard D. Straus and Comtrol Corporation.         *
; *   All rights reserved.  Use or reproduction by any means of any       *
; *   portion of this code contrary to the developers license agreement   *
; *   is expressly prohibited.                                            *
; ************************************************************************/
; *                                                                       *
; *   Notes                                                               *
; *   -----                                                               *
; *   This file contains the startup routines, assembly language          *
; *   functions and interrupt entry code for the Turbo C++ version        *
; *   of the Comtrol Hostess i Programmer's Toolkit Control Program.      *
; *                                                                       *
; *0**********************************************************************/

.286
.MODEL tiny

.xlist
.list

page

public  __acrtused              ; trick to force in startup for Microsoft
        __acrtused = 9876h          ; ignored in Turbo C++
        .xlist
        include cpmain.equ          ; include file for constants and definitions
        .list

        extrn   _tmr_isr:near       ; Timer (1) Interrupt Service Routine
        extrn   _cp_acp:near        ; Main Hostess i Control Program
        extrn   _cp_ini:near        ; Hostess i initialization function
        extrn   _ho_isr:near        ; Host interrupt service routine (INT1)
        extrn   _tbe_isr:near
        extrn   _esc_isr:near
        extrn   _rxf_isr:near
        extrn   _src_isr:near

;dgroup  group   _DATA, _BSS

;_DATA   segment word public 'DATA'
;_DATA   ends

;_BSS    segment word public 'BSS'
;_BSS    ends
;_TEXT   segment word public 'CODE'

         assume cs:_TEXT, ds:_TEXT

stack_top        =       0F000H          ; stack at top of segment


; data structure of the first 80H bytes of the high page of Hostess i dual-
; ported RAM.

.CODE

                 org     0
i_flag           dw      ?             ; 0h
boot_flag        dw      ?             ; 2h
cfg_map          dw      ?             ; 4h
fw_release       db      8 dup (?)     ; 6h
sw_release       db      8 dup (?)     ; Eh
reserved         db      4 dup (?)     ; 16h
dram_map         dd      ?             ; 1Ah
scc_map          dd      ?             ; 1Eh
board_id         dd      ?             ; 22h
invalid_int      db      ?             ; 26h
open             db      73 dup (?)    ; 27 - 6Fh
s2c_flag         dw      ?             ; 70h
s2c_data1        dw      ?             ; 72h
s2c_data2        dw      ?             ; 74h
x2x_data         dw      ?             ; 76h
c2s_flag         dw      ?             ; 78h
c2s_data1        dw      ?             ; 7Ah
c2s_data2        dw      ?             ; 7Ch
halt_loop        dw      ?             ; 7Eh


;************************************************************************
;*  name:       cpmain                                                  *
;*  purpose:    initial setup for Hostess i control program             *
;*  call:       loaded from host, activated by host interrupting        *
;*              Hostess i                                               *
;*  parameters: none.                                                   *
;*  returns:    none.                                                   *
;*  method:     sets up segment registers, timer interrupt vector and   *
;*      all port interrupt vectors, then calls the C language control   *
;*      program to begin serial and host communications.  The control   *
;*      program never returns to cpmain.                                *
;*  functions:  vector_init     initialize 8530 SCC interrupt vectors   *
;*              _cp_acp         main monitor loop for Hostess i         *
;*  notes:      the first executable instruction must be at 80H in the  *
;*      high page of the Hostess i dual-ported memory.  Care must be    *
;*      teken to load it at the proper location.                        *
;************************************************************************

           public  start
           public  cpmain
cpmain     proc    near

        ; Initialize segment registers

IFDEF INVOKE_TD
        int     27h                     ; invoke Turbo Debugger remote
ENDIF
start:
        cli                             ; disable interrupts

        mov     ax, cs                  ; establish data segment = CS
        mov     ds, ax
        mov     ss, ax                  ; establish a program stack
        mov     sp, stack_top           ; starts at top of memory

        xor     ax, ax                  ; clear ES register
        mov     es, ax

        mov     bx, TIMER_INTR_NO * 4   ; setup timer interrupt vector
        mov     ax, offset timer_isr
        mov     es:[bx], ax
        mov     ax,cs
        mov     es:[bx+2], ax

        mov     ax, base_vector * 4     ; get offset of SCC vectors
        mov     di, ax
        xor     ax, ax
        mov     es, ax                  ; ES:DI=> destination
        mov     si, offset vector_tbl   ; SI=> vector table
        mov     cx, (vector_tbl_end - vector_tbl) / 2 ; words in vector table
        mov     bx, cs                  ; setup segment address

vector_init_10:
        movsw                   ; move vector offset
        mov     ax, bx
        stosw
        add     di, 4           ; skip vector entry
        loop    vector_init_10  ; continue ...

        xor     ax, ax          ; clear AX
        mov     es, ax          ; set segment = 0000 (interrupt vectors)
        mov     bx, INT1_type * 4     ; address of INT1 (host) interrupt vector
        mov     ax, offset ho_ise
        mov     es:[bx], ax     ; store offset of host interrupt service entry
        mov     ax, cs
        mov     es:[bx+2], ax   ; store segment of interrupt service entry

        mov     ax, 1000H
        mov     es, ax

        sti                     ; restore interrupt system

        call    _cp_acp         ; call the main C language control program
                                                        ; which never returns
main99:
        jmp     main99          ; loop forever

cpmain    endp



;*******************************************************************
;  name:        _timer_start
;  purpose:     initialize and start timer 1 for 0.1 second interrupts
;  call:                timer_start();
;  parameters:  none.
;  returns:     none.
;  method:      sets the registers timer 1 to generate 0.1 second
;               interrupts, then activates the timer.
;*******************************************************************

        public  _timer_start
_timer_start    proc

                push    ax
                push    bx
                push    dx
                push    es

                pushf
                cli

                mov     bx,TIMER_INTR_NO * 4    ; setup timer intr vector
                mov     ax,offset timer_isr
                mov     es:[bx],ax
                mov     ax,cs
                mov     es:[bx+2],ax

                mov     dx, TIM_CTL
                mov     al, SELECT_TIMER         ; enable timer1
                out     dx, al
                mov     dx, TIMER_PORT
                mov     al, 12H                  ; 25/sec LSB
                out     dx, al
                mov     al, 7AH                  ; 25/sec MSB
                out     dx,al

                popf

                pop     es
                pop     dx
                pop     bx
                pop     ax
                ret

_timer_start    endp



;*******************************************************************
;  name:        timer_isr
;  purpose:     interface for the 'C' language timer interrupt service
;               routine
;  parameters:  none.
;  returns:     none.
;  method:      saves all the relevant registers, then calls the
;               'C' language timer interrupt service routine.  When
;               the ISR returns, restores the registers and exits the
;               interrupt.
;*******************************************************************

        public timer_isr
timer_isr       proc
        pusha                           ; save registers
        push    es                      ; for debugger

        mov     dx,INTCTL               ; end of interrupt
        mov     al,20h
        out     dx,al

        call    _tmr_isr                ; call 'C' routine

        pop     es
        popa                            ; recover registers
        iret                                                    ; return from interrupt

timer_isr       endp



;   host interrupt service routine entry function

        public  ho_ise
ho_ise proc
        pusha                   ; save all registers
        push    es              ; including ES, not saved by pusha

        call    _ho_isr         ; call the C host interrupt handler

        mov     dx, INTCTL      ; write End-of-Interrupt (EOI) to the
        mov     al, EOI         ; 8259 PIC
        out     dx, al

        pop     es              ; pop saved registers
        popa
        iret
ho_ise endp


;************************************************************************
;  name:        _dint
;  purpose:     disable the interrupt system, and return the current state
;               of the flags register.
;  call:        s = dint();
;  parameters:  none.
;  returns:
;       int s;  the state of the flags register before the interrupts were
;               disabled.
;  method:      pushes the flags register into AX, then clears the inter-
;               rupt flag.  Returns the AX register to the caller.
;************************************************************************

        public  _dint
_dint   proc

        pushf                           ; get current state of flags
        pop     ax                      ; return in AX
        cli                             ; disable interrupts
        ret

_dint   endp


;************************************************************************
;  name:        _eint
;  purpose:     restore the interrupt system to its state before a dint.
;  call:        eint(s);
;  parameters:
;       int s;  state to store in the flags register
;  returns:     none.
;  method:      stores the parameter into the flags register.
;************************************************************************

;eistr           struct
;eibp    dw      ?                       ; old frame pointer
;eira    dw      ?                       ; return address
;eiflgs  dw      ?                       ; flags parameter
;eistr           ends

        public  _eint
_eint   proc

        push    bp              ; save old frame register
        mov     bp, sp          ; develop new one
        mov     ax, 4[bp]       ; get parameter
        push    ax              ; to stack
        popf                    ; tramsfer to flags register
        pop     bp              ; restore old frame register
        ret                     ; return to caller

_eint   endp


;************************************************************************
;  name:        _debug
;  purpose:     enters the Hostess i on-board debugger.  returns to the
;               caller when the debugger gets the instruction "G[O]".
;  call:        from 'C':  debug();
;               from asm:  call _debug
;  parameters:  none.
;  returns:     none.
;  method:      masks off interrupts from the 8259A PIC, then executes
;               an INT 20H to enter the ROM Debugger.  Upon returning
;               from the debugger, restores the PIC mask, and exits.
;************************************************************************

        public  _debug
_debug          proc

        push    dx
        mov     dx, INTMASK             ; 8259A mask register
        in      al, dx                  ; get current value
        push    ax                      ; save it
        mov     al, 0FFH                ; mask off interrupts before debug
        out     dx, al
        int     20H                     ; enter the Hostess i debugger
        pop     ax
        mov     dx, INTMASK             ; 8259A mask register
        out     dx, al                  ; restore to its former value
        pop     dx
        ret                             ; return to the caller

_debug          endp

;************************************************************************
;  name:        _wreg
;  purpose:     write a value to an 8530 register
;  call:        wreg(port, regno, value);
;  parameters:
;       int port;   8530 port base address
;       int regno;  the register number to use
;       int value;  data to write to the register
;  returns:     none.
;  method:      first writes the register select number to the desired
;       8530 port, then writes the data byte to the same port.  Appro-
;       priate delays are inserted between 8530 writes.
;************************************************************************

;wr     struct
;wrbp   dw      ?       ; 0
;wrra   dw      ?       ; 2
;wrport dw      ?       ; 4
;wrregn dw      ?       ; 6
;wrvalu dw      ?       ; 8
;wr     ends

        public  _wreg
_wreg   proc

        push    bp                      ; save old frame register
        mov     bp, sp                  ; develop new one
        push    dx                      ; save destroyed register
        mov     dx, [bp + 4]            ; get port address
        mov     al, [bp + 6]            ; get register number

        pushf                                                   ; save current interrupt state
        cli                                                             ; disable interrupt system

        out     dx, al                  ; write register select
        mov     al, [bp + 8]            ; get the value for the register
        out     dx, al                  ; write to the register

        popf                                                    ; restore former interrupt state

        pop     dx
        pop     bp
        ret

_wreg   endp


;************************************************************************
;  name:        _rreg
;  purpose:     read the contents of an 8530 read register
;  call:        val = rreg(port, regno);
;  parameters:
;       int port;   port address
;       int regno;  register selector
;  returns:
;       int val;    contents of read register
;  method:      writes the register selector to the given 8530 port, then
;       reads the same port for the contents of the selected register.
;       Appropriate delays are inserted between the 8530 write and read.
;************************************************************************

;rr     struct
;rrbp   dw      ?       ; 0
;rrra   dw      ?       ; 2
;rrport dw      ?       ; 4
;rrregn dw      ?       ; 6
;rr     ends

        public  _rreg
_rreg   proc
        push    bp                      ; save old frame pointer
        mov     bp, sp                  ; develop new one
        push    dx                      ; save destroyed register
        mov     dx, [bp + 4]            ; get port address
        mov     al, [bp + 6]            ; get register number

        pushf                                                   ; save current interrupt state
        cli                                                         ; disable interrupt system

        out     dx, al                  ; write register select
        in      al, dx                  ; read value from selected register

        popf                                                    ; restore former interrupt state

        sub     ah, ah                  ; clear high byte
        pop     dx                      ; restore registers
        pop     bp
        ret
_rreg   endp


;************************************************************************
;  name:        _intdiv
;  purpose:     replace the normal 'C' integer divide arithmetic with a
;           function to avoid the error checking.
;  call:        result = intdiv(dividend, divisor);
;  parameters:
;       unsigned long dividend;  the number to be divided
;       unsigned int  divisor;   the number to divide by
;  returns:
;       unsigned int  result;    the result of the division
;  method:      does a normal, unsigned integer divide of one number by
;       another.
;  notes:       normal 'C' integer division pulls in from the library
;       error checking routines and interrupt handlers to handle divide
;       by zero occurrences.  Since the board does limited arithmetic,
;       we do the divide directly, and with no checking.
;************************************************************************

;idstr           struct
;idbp    dw      ?                       ; old frame pointer
;idra    dw      ?                       ; return address
;idnum1  dd      ?                       ; first parameter
;idnum2  dw      ?                       ; second parameter
;idstr           ends

        public  _intdiv                 ; integer divide
_intdiv         proc

        push    bp                      ; save old frame pointer
        mov     bp, sp                  ; generate frame pointer
        mov     ax, 4[bp]               ; load up dividend
        mov     dx, 6[bp]
        div     word ptr 8[bp]          ; quotient to AX
        pop     bp
        ret

_intdiv         endp


;************************************************************************
;  name:        _intmod
;  purpose:     replace the normal 'C' language '%' operator
;  call:        modulo = intmod(num1, num2);
;  parameters:
;       unsigned long num1;  the number of which to take the modulo
;       unsigned int  num2;  the divisor
;  returns:
;       unsigned int  modulo;  the result of num1 % num2
;  notes:       normal 'C' integer division pulls in from the library
;       error checking routines and interrupt handlers to handle divide
;       by zero occurrences.  Since the board does limited arithmetic,
;       we do the divide directly, and with no checking.
;************************************************************************

;imstr   struct
;imbp    dw      ?                       ; old frame pointer
;imra    dw      ?                       ; return address
;imnum1  dd      ?                       ; first parameter
;imnum2  dw      ?                       ; second parameter
;imstr   ends

        public  _intmod                 ; modulo computation
_intmod         proc

        push    bp                      ; save old frame pointer
        mov     bp, sp                  ; generate frame pointer
        push    dx
        mov     ax, 4[bp]               ; load up dividend
        mov     dx, 6[bp]
        div     word ptr 8[bp]          ; quotient to AX, remainder to DX
        mov     ax, dx                  ; remainder to return
        pop     dx                      ; restore registers
        pop     bp
        ret

_intmod         endp



;  Interrupt Service Routines
;  --------------------------


;  LINE 0

line00_RCA  proc            ; Receive Character Available
        pusha                   ; save registers
        mov     ax, line00      ; and setup interrupt type
        jmp     RXF_isr         ; do common processing ...
line00_RCA  endp

line00_TBE  proc            ; Transmit Buffer Empty
        pusha                   ; save registers
        mov     ax, line00      ; and setup line table #
        jmp     TBE_isr         ; do common processing ...
line00_TBE  endp

line00_ESC  proc            ; External/Status Change
        pusha                   ; save registers
        mov     ax, line00      ; and setup line table #
        jmp     ESC_isr         ; do common processing ...
line00_ESC  endp

line00_SRC  proc            ; Special Receive Condition
        pusha                   ; save registers
        mov     ax, line00      ; and setup interrupt type
        jmp     SRC_isr         ; do common processing ...
line00_SRC  endp

;
;  LINE 1
;
line01_RCA  proc            ; Receive Character Available
        pusha                   ; save registers
        mov     ax, line01      ; and setup interrupt type
        jmp     RXF_isr         ; do common processing ...
line01_RCA  endp

line01_TBE  proc            ; Transmit Buffer Empty
        pusha                   ; save registers
        mov     ax, line01      ; and setup line table #
        jmp     TBE_isr         ; do common processing ...
line01_TBE  endp

line01_ESC  proc            ; External/Status Change
        pusha                   ; save registers
        mov     ax, line01      ; and setup line table #
        jmp     ESC_isr         ; do common processing ...
line01_ESC  endp

line01_SRC  proc            ; Special Receive Condition
        pusha                   ; save registers
        mov     ax, line01      ; and setup interrupt type
        jmp     SRC_isr         ; do common processing ...
line01_SRC  endp

;
;  LINE 2
;
line02_RCA  proc            ; Receive Character Available
        pusha                   ; save registers
        mov     ax, line02      ; and setup interrupt type
        jmp     RXF_isr         ; do common processing ...
line02_RCA  endp

line02_TBE  proc            ; Transmit Buffer Empty
        pusha                   ; save registers
        mov     ax, line02      ; and setup line table #
        jmp     TBE_isr         ; do common processing ...
line02_TBE  endp

line02_ESC  proc            ; External/Status Change
        pusha                   ; save registers
        mov     ax, line02      ; and setup line table #
        jmp     ESC_isr         ; do common processing ...
line02_ESC  endp

line02_SRC  proc            ; Special Receive Condition
        pusha                   ; save registers
        mov     ax, line02      ; and setup interrupt type
        jmp     SRC_isr         ; do common processing ...
line02_SRC  endp

;
;  LINE 3
;
line03_RCA  proc            ; Receive Character Available
        pusha                   ; save registers
        mov     ax, line03      ; and setup interrupt type
        jmp     RXF_isr         ; do common processing ...
line03_RCA  endp

line03_TBE  proc            ; Transmit Buffer Empty
        pusha                   ; save registers
        mov     ax, line03      ; and setup line table #
        jmp     TBE_isr         ; do common processing ...
line03_TBE  endp

line03_ESC  proc            ; External/Status Change
        pusha                   ; save registers
        mov     ax, line03      ; and setup line table #
        jmp     ESC_isr         ; do common processing ...
line03_ESC  endp

line03_SRC  proc            ; Special Receive Condition
        pusha                   ; save registers
        mov     ax, line03      ; and setup interrupt type
        jmp     SRC_isr         ; do common processing ...
line03_SRC  endp

;
;  LINE 4
;
line04_RCA  proc            ; Receive Character Available
        pusha                   ; save registers
        mov     ax, line04      ; and setup interrupt type
        jmp     RXF_isr         ; do common processing ...
line04_RCA  endp

line04_TBE  proc            ; Transmit Buffer Empty
        pusha                   ; save registers
        mov     ax, line04      ; and setup line table #
        jmp     TBE_isr         ; do common processing ...
line04_TBE  endp

line04_ESC  proc            ; External/Status Change
        pusha                   ; save registers
        mov     ax, line04      ; and setup line table #
        jmp     ESC_isr         ; do common processing ...
line04_ESC  endp

line04_SRC  proc            ; Special Receive Condition
        pusha                   ; save registers
        mov     ax, line04      ; and setup interrupt type
        jmp     SRC_isr         ; do common processing ...
line04_SRC  endp

;
;  LINE 5
;
line05_RCA  proc            ; Receive Character Available
        pusha                   ; save registers
        mov     ax, line05      ; and setup interrupt type
        jmp     RXF_isr         ; do common processing ...
line05_RCA  endp

line05_TBE  proc            ; Transmit Buffer Empty
        pusha                   ; save registers
        mov     ax, line05      ; and setup line table #
        jmp     TBE_isr         ; do common processing ...
line05_TBE  endp

line05_ESC  proc            ; External/Status Change
        pusha                   ; save registers
        mov     ax, line05      ; and setup line table #
        jmp     ESC_isr         ; do common processing ...
line05_ESC  endp

line05_SRC  proc            ; Special Receive Condition
        pusha                   ; save registers
        mov     ax, line05      ; and setup interrupt type
        jmp     SRC_isr         ; do common processing ...
line05_SRC  endp

;
;  LINE 6
;
line06_RCA  proc            ; Receive Character Available
        pusha                   ; save registers
        mov     ax, line06      ; and setup interrupt type
        jmp     RXF_isr         ; do common processing ...
line06_RCA  endp

line06_TBE  proc            ; Transmit Buffer Empty
        pusha                   ; save registers
        mov     ax, line06      ; and setup line table #
        jmp     TBE_isr         ; do common processing ...
line06_TBE  endp

line06_ESC  proc            ; External/Status Change
        pusha                   ; save registers
        mov     ax, line06      ; and setup line table #
        jmp     ESC_isr         ; do common processing ...
line06_ESC  endp

line06_SRC  proc            ; Special Receive Condition
        pusha                   ; save registers
        mov     ax, line06      ; and setup interrupt type
        jmp     SRC_isr         ; do common processing ...
line06_SRC  endp

;
;  LINE 7
;
line07_RCA  proc            ; Receive Character Available
        pusha                   ; save registers
        mov     ax, line07      ; and setup interrupt type
        jmp     RXF_isr         ; do common processing ...
line07_RCA  endp

line07_TBE  proc            ; Transmit Buffer Empty
        pusha                   ; save registers
        mov     ax, line07      ; and setup line table #
        jmp     TBE_isr         ; do common processing ...
line07_TBE  endp

line07_ESC  proc            ; External/Status Change
        pusha                   ; save registers
        mov     ax, line07      ; and setup line table #
        jmp     ESC_isr         ; do common processing ...
line07_ESC  endp

line07_SRC  proc            ; Special Receive Condition
        pusha                   ; save registers
        mov     ax, line07      ; and setup interrupt type
        jmp     SRC_isr         ; do common processing ...
line07_SRC  endp

;
;  LINE 8
;
line08_RCA  proc            ; Receive Character Available
        pusha                   ; save registers
        mov     ax, line08      ; and setup interrupt type
        jmp     RXF_isr         ; do common processing ...
line08_RCA  endp

line08_TBE  proc            ; Transmit Buffer Empty
        pusha                   ; save registers
        mov     ax, line08      ; and setup line table #
        jmp     TBE_isr         ; do common processing ...
line08_TBE  endp

line08_ESC  proc            ; External/Status Change
        pusha                   ; save registers
        mov     ax, line08      ; and setup line table #
        jmp     ESC_isr         ; do common processing ...
line08_ESC  endp

line08_SRC  proc            ; Special Receive Condition
        pusha                   ; save registers
        mov     ax, line08      ; and setup interrupt type
        jmp     SRC_isr         ; do common processing ...
line08_SRC  endp

;
;  LINE 9
;
line09_RCA  proc            ; Receive Character Available
        pusha                   ; save registers
        mov     ax, line09      ; and setup interrupt type
        jmp     RXF_isr         ; do common processing ...
line09_RCA  endp

line09_TBE  proc            ; Transmit Buffer Empty
        pusha                   ; save registers
        mov     ax, line09      ; and setup line table #
        jmp     TBE_isr         ; do common processing ...
line09_TBE  endp

line09_ESC  proc            ; External/Status Change
        pusha                   ; save registers
        mov     ax, line09      ; and setup line table #
        jmp     ESC_isr         ; do common processing ...
line09_ESC  endp

line09_SRC  proc            ; Special Receive Condition
        pusha                   ; save registers
        mov     ax, line09      ; and setup interrupt type
        jmp     SRC_isr         ; do common processing ...
line09_SRC  endp

;
;  LINE 10
;
line10_RCA  proc            ; Receive Character Available
        pusha                   ; save registers
        mov     ax, line10      ; and setup interrupt type
        jmp     RXF_isr         ; do common processing ...
line10_RCA  endp

line10_TBE  proc            ; Transmit Buffer Empty
        pusha                   ; save registers
        mov     ax, line10      ; and setup line table #
        jmp     TBE_isr         ; do common processing ...
line10_TBE  endp

line10_ESC  proc            ; External/Status Change
        pusha                   ; save registers
        mov     ax, line10      ; and setup line table #
        jmp     ESC_isr         ; do common processing ...
line10_ESC  endp

line10_SRC  proc            ; Special Receive Condition
        pusha                   ; save registers
        mov     ax, line10      ; and setup interrupt type
        jmp     SRC_isr         ; do common processing ...
line10_SRC  endp

;
;  LINE 11
;
line11_RCA  proc            ; Receive Character Available
        pusha                   ; save registers
        mov     ax, line11      ; and setup interrupt type
        jmp     RXF_isr         ; do common processing ...
line11_RCA  endp

line11_TBE  proc            ; Transmit Buffer Empty
        pusha                   ; save registers
        mov     ax, line11      ; and setup line table #
        jmp     TBE_isr         ; do common processing ...
line11_TBE  endp

line11_ESC  proc            ; External/Status Change
        pusha                   ; save registers
        mov     ax, line11      ; and setup line table #
        jmp     ESC_isr         ; do common processing ...
line11_ESC  endp

line11_SRC  proc            ; Special Receive Condition
        pusha                   ; save registers
        mov     ax, line11      ; and setup interrupt type
        jmp     SRC_isr         ; do common processing ...
line11_SRC  endp

;
;  LINE 12

line12_RCA  proc            ; Receive Character Available
        pusha                   ; save registers
        mov     ax, line12      ; and setup interrupt type
        jmp     RXF_isr         ; do common processing ...
line12_RCA  endp

line12_TBE  proc            ; Transmit Buffer Empty
        pusha                   ; save registers
        mov     ax, line12      ; and setup line table #
        jmp     TBE_isr         ; do common processing ...
line12_TBE  endp

line12_ESC  proc            ; External/Status Change
        pusha                   ; save registers
        mov     ax, line12      ; and setup line table #
        jmp     ESC_isr         ; do common processing ...
line12_ESC  endp

line12_SRC  proc            ; Special Receive Condition
        pusha                   ; save registers
        mov     ax, line12      ; and setup interrupt type
        jmp     SRC_isr         ; do common processing ...
line12_SRC  endp

;
;  LINE 13
;
line13_RCA  proc            ; Receive Character Available
        pusha                   ; save registers
        mov     ax, line13      ; and setup interrupt type
        jmp     RXF_isr         ; do common processing ...
line13_RCA  endp

line13_TBE  proc            ; Transmit Buffer Empty
        pusha                   ; save registers
        mov     ax, line13      ; and setup line table #
        jmp     TBE_isr         ; do common processing ...
line13_TBE  endp

line13_ESC  proc            ; External/Status Change
        pusha                   ; save registers
        mov     ax, line13      ; and setup line table #
        jmp     ESC_isr         ; do common processing ...
line13_ESC  endp

line13_SRC  proc            ; Special Receive Condition
        pusha                   ; save registers
        mov     ax, line13      ; and setup interrupt type
        jmp     SRC_isr         ; do common processing ...
line13_SRC  endp

;
;  LINE 14
;
line14_RCA  proc            ; Receive Character Available
        pusha                   ; save registers
        mov     ax, line14      ; and setup interrupt type
        jmp     RXF_isr         ; do common processing ...
line14_RCA  endp

line14_TBE  proc            ; Transmit Buffer Empty
        pusha                   ; save registers
        mov     ax, line14      ; and setup line table #
        jmp     TBE_isr         ; do common processing ...
line14_TBE  endp

line14_ESC  proc            ; External/Status Change
        pusha                   ; save registers
        mov     ax, line14      ; and setup line table #
        jmp     ESC_isr         ; do common processing ...
line14_ESC  endp

line14_SRC  proc            ; Special Receive Condition
        pusha                   ; save registers
        mov     ax, line14      ; and setup interrupt type
        jmp     SRC_isr         ; do common processing ...
line14_SRC  endp

;
;  LINE 15
;
line15_RCA  proc            ; Receive Character Available
        pusha                   ; save registers
        mov     ax, line15      ; and setup interrupt type
        jmp     RXF_isr         ; do common processing ...
line15_RCA  endp

line15_TBE  proc            ; Transmit Buffer Empty
        pusha                   ; save registers
        mov     ax, line15      ; and setup line table #
        jmp     TBE_isr         ; do common processing ...
line15_TBE  endp

line15_ESC  proc            ; External/Status Change
        pusha                   ; save registers
        mov     ax, line15      ; and setup line table #
        jmp     ESC_isr         ; do common processing ...
line15_ESC  endp

line15_SRC  proc            ; Special Receive Condition
        pusha                   ; save registers
        mov     ax, line15      ; and setup interrupt type
        jmp     SRC_isr         ; do common processing ...
line15_SRC  endp

;
;  Common Interrupt Service Routines
;  ---------------------------------
;
;  Transmit Buffer Empty Interrupt Service Routine
;

TBE_isr proc
        push    es              ; insure ES is saved
        push    ax              ; interrupting line number is parameter
        call    _tbe_isr        ; C language ISR
        add     sp, 2
        jmp     isr_ret         ; common exit ...
TBE_isr endp

;
;  External Status Change Interrupt Service Routine
;

ESC_isr proc
        push    es              ; insure ES is saved
        push    ax              ; interrupting line number is parameter
        call    _esc_isr        ; C language ISR
        add     sp, 2
        jmp     isr_ret         ; common exit ...
ESC_isr endp

;
;  Receive Character Available Interrupt Service Routine
;

RXF_isr proc
        push    es              ; insure ES is saved
        push    ax              ; interrupting line number is parameter
        call    _rxf_isr        ; C language ISR
        add     sp, 2
        jmp     isr_ret         ; common exit ...
RXF_isr endp

;
;  Special Receive Condition Interrupt Service Routine
;

SRC_isr proc
        push    es              ; insure ES is saved
        push    ax              ; interrupting line number is parameter
        call    _src_isr        ; C language ISR
        add     sp, 2
        jmp     isr_ret
SRC_isr endp

;
;  name: isr_ret
;  function: common Interrupt Service Routine exit processing
;
;  enter: AX = chip port number
;         registers and flags on the stack
;  exit:  to interrupted routine

isr_ret proc
        mov     dx, ax          ; get port number to dx
        sub     al, al          ; insure WR0
        out     dx, al

        mov     al, 38H         ; clear highest IUS
        out     dx, al

        mov     dx, INTCTL      ; Hostess i SCC interrupts go through PIC
        mov     al, EOI         ; and must receive an EOI
        out     dx, al

        pop     es              ; for debugger
        popa                    ; restore registers
        iret
isr_ret endp

;************************************************************************
;*                                                                      *
;*   Interrupt Vector Table                                             *
;*   ----------------------                                             *
;*                                                                      *
;*   To load and set up the interrupt vectors, the cpmain program copies*
;*   this table into the interrupt vector table. The interrupt numbers  *
;*   start from 80H, thus the first interrupt vector is stored at the   *
;*   address 0000:0200 which is 80H * 4 (bytes per vector).             *
;*   The vectors are stored with one vector space between them, as the  *
;*   8530 chip only uses even numbered interrupts.  The segment for     *
;*   each of them is the Code Segment (CS).                             *
;*                                                                      *
;************************************************************************

vector_tbl  equ $

        dw      line01_TBE
        dw      line01_ESC
        dw      line01_RCA
        dw      line01_SRC

        dw      line00_TBE
        dw      line00_ESC
        dw      line00_RCA
        dw      line00_SRC

        dw      line03_TBE
        dw      line03_ESC
        dw      line03_RCA
        dw      line03_SRC

        dw      line02_TBE
        dw      line02_ESC
        dw      line02_RCA
        dw      line02_SRC

        dw      line05_TBE
        dw      line05_ESC
        dw      line05_RCA
        dw      line05_SRC

        dw      line04_TBE
        dw      line04_ESC
        dw      line04_RCA
        dw      line04_SRC

        dw      line07_TBE
        dw      line07_ESC
        dw      line07_RCA
        dw      line07_SRC

        dw      line06_TBE
        dw      line06_ESC
        dw      line06_RCA
        dw      line06_SRC

        dw      line09_TBE
        dw      line09_ESC
        dw      line09_RCA
        dw      line09_SRC

        dw      line08_TBE
        dw      line08_ESC
        dw      line08_RCA
        dw      line08_SRC

        dw      line11_TBE
        dw      line11_ESC
        dw      line11_RCA
        dw      line11_SRC

        dw      line10_TBE
        dw      line10_ESC
        dw      line10_RCA
        dw      line10_SRC

        dw      line13_TBE
        dw      line13_ESC
        dw      line13_RCA
        dw      line13_SRC

        dw      line12_TBE
        dw      line12_ESC
        dw      line12_RCA
        dw      line12_SRC

        dw      line15_TBE
        dw      line15_ESC
        dw      line15_RCA
        dw      line15_SRC

        dw      line14_TBE
        dw      line14_ESC
        dw      line14_RCA
        dw      line14_SRC

vector_tbl_end  equ     $

_TEXT   ends

        end

