/**********************************************************************
*  File:        CP.C                                                  *
*  Purpose:     COMTROL Hostess i board Demo Control Program Main Loop*
*               and Initialization functions.                         *
*  Date:        September 20, 1990                                    *
*  Author:      Howard D. Straus                                      *
*  Release: 1.00 - Original release                                   *
***********************************************************************
*                         COPYRIGHT NOTICE                            *
*=====================================================================*
*   Copyright 1990, 1991 by Howard D. Straus and Comtrol Corporation. *
*   All rights reserved.  Use or reproduction by any means of any     *
*   portion of this code contrary to the developers license agreement *
*   is expressly prohibited.                                          *
**********************************************************************/

#include "compiler.h"

#include <dos.h>

#ifdef  TURBO_C
#include <mem.h>
#endif

#include "hostess.h"
#include "proto.h"

int     curr_line;              /* current line */

MG_     host_message_buffer;    /* host message buffer */
MG_     hostess_message_buffer; /* Hostess i message buffer */

DV_     dct[N_PORTS];           /* device control tables */
LC_     lct[N_PORTS];           /* line control tables */
DV_    *dvad[N_PORTS];          /* device control table addresses */

TIME    curr_time = 0;          /* current time, in seconds */
long    elapsed_ticks = 0;      /* total elapsed ticks */

MB_     far     *mb;            /* mailbox address */

QC_     host_queue;             /* host queue control block */
QC_     free_pool;              /* free pool of buffers */
int     n_free_buffers;         /* free buffer count */
int             host_interrupt_flag = 0; /* host generated INT1 */

int far *flag_ptr;              /* pointer to response word */
int     time_pending = FALSE;   /* status report in queue */

#ifndef TURBO_C

union   {
        unsigned int    so[2];
        void    far     *ptr;
} f_p;

#endif

/*
        PC-LSC Main Program module, called from assembly language startup
*/
void
cp_acp()
{   static  int     ln = 0;         /* line number */

        cp_ini();                       /* initialize system */

        for (;;)                        /* forever loop */
        {   dv_adv(dvad[ln]);           /* check device conditions */
                lc_alc(dvad[ln]->dv_lcad);  /* check line activity */
                if (host_interrupt_flag)    /* if host interrupted Hostess */
                {   ho_chi();               /* process host input */
                        --host_interrupt_flag;  /* decrement pending messages */
                }
                if (++ln >= N_PORTS)        /* check next line */
                {   ho_aho();               /* check host activity regularly */
                        ln = 0;                 /* wrap around */
                }
        }
}


/*
        System Initialization function
*/
void
cp_ini()
{   int             ln;                 /* line loop counter */
        LC_             *lc;                /* line control pointer */
        LC_             *last_lc;           /* for chaining */
        DV_             *dv;                /* device control pointer */
        DV_             *last_dv;           /* for chaining */

        mb = MK_FP(HIGH_PAGE_SEG, MAILBOX); /* generate the mailbox pointer */
        fmemset((char far *)mb, 0, sizeof(MB_)); /* clear the mailbox */

        mb->mb_homg = (MG_ far *)&host_message_buffer;
                                                                                /* point to host message buffer */
        mb->mb_ulmg = (MG_ far *)&hostess_message_buffer;
                                                                                /* point to Hostess message buffer */

        bq_qci((QC_ far *)&host_queue);     /* initialize host queue */

        for (ln = 0; ln < N_PORTS; ln++)    /* for each line */
        {   dv = dvad[ln] = &dct[ln];       /* generate a device table pointer */
                lc = &lct[ln];                  /* amd a line control table pointer */
                memset(lc, 0, sizeof(LC_));     /* clear the line block */
                memset(dv, 0, sizeof(DV_));     /* clear the device block */
                lc->lc_lnno = ln;               /* save line number */

                if (!ln)                        /* only for host access */
                {   mb->mb_dvad = (DV_ far *)dv; /* far pointer to first DCT */
                }
                else
                {   last_lc->lc_next = lc;      /* thread blocks together */
                        last_dv->dv_next = dv;
                }
                lc->lc_dvad = dv;               /* connect dv and lc tables */
                dv->dv_lcad = lc;

                last_lc = lc;                   /* save for next loop iteration */
                last_dv = dv;

                bq_qci((QC_ far *)&lc->lc_txqc); /* initialize line's Tx queue */
        }

        bq_ini();                           /* initialize buffer pool */
        tm_ini();                           /* initialize the timer */
        curr_line = 1;                      /* default line */
        host_interrupt_flag = 0;            /* clear pending interrupt ctr */
        flag_ptr = (int far *)MK_FP(0x1000, 0);
                                                                                /* point to interaction flag word */

        *flag_ptr = TOOLKIT_RUNNING_VALUE;  /* let host know we're ready */
}




                                        /*  *****  END OF FILE  *****  */
