/**********************************************************************
*  File Name: DV.C                                                    *
*  Purpose:   Comtrol Hostess i Device Control Functions for 8530     *
*       SIO Controller Chips.                                         *
*  Date:      January 1, 1991                                         *
*  Release: 1.00 - Original release                                   *
***********************************************************************
*                         COPYRIGHT NOTICE                            *
*=====================================================================*
*   Copyright 1990, 1991 by Howard D. Straus and Comtrol Corporation. *
*   All rights reserved.  Use or reproduction by any means of any     *
*   portion of this code contrary to the developers license agreement *
*   is expressly prohibited.                                          *
***********************************************************************/

#include "compiler.h"

#include <dos.h>

#include "hostess.h"
#include "proto.h"
#include "externs.h"

#define RR(x)      rreg(dv->dv_cmdp, x)

#define WR0(x)     wreg(dv->dv_cmdp, 0, x)
#define WR1(x)     wreg(dv->dv_cmdp, 1, x)
#define WR2(x)     wreg(dv->dv_cmdp, 2, x)
#define WR3(x)     wreg(dv->dv_cmdp, 3, x)
#define WR4(x)     wreg(dv->dv_cmdp, 4, x)
#define WR5(x)     wreg(dv->dv_cmdp, 5, x)
#define WR6(x)     wreg(dv->dv_cmdp, 6, x)
#define WR7(x)     wreg(dv->dv_cmdp, 7, x)
#define WR8(x)     wreg(dv->dv_cmdp, 8, x)
#define WR9(x)     wreg(dv->dv_cmdp, 9, x)
#define WR10(x)    wreg(dv->dv_cmdp, 10, x)
#define WR11(x)    wreg(dv->dv_cmdp, 11, x)
#define WR12(x)    wreg(dv->dv_cmdp, 12, x)
#define WR13(x)    wreg(dv->dv_cmdp, 13, x)
#define WR14(x)    wreg(dv->dv_cmdp, 14, x)
#define WR15(x)    wreg(dv->dv_cmdp, 15, x)

#define WRCH(x)    OUTP(dv->dv_datp, x)
#define RDCH()     INP(dv->dv_datp)

#define dv_gr0(dv)  (dv->dv_lrr0 = RR(0))
#define dv_mr5(dv)  (dv->dv_dtr | dv->dv_rts | dv->dv_txe | bpc[dv->dv_bits])

/* Base (command) port addresses for the Hostess i 8530 ports */
int     base_port[] = { PORT0,  PORT1,  PORT2,  PORT3, PORT4, PORT5,
                                                PORT6,  PORT7,  PORT8,  PORT9, PORT10, PORT11,
                                                PORT12, PORT13, PORT14, PORT15 };

/* bits per character specifiers for WR3 and WR5 */
int     bpc[] = { 0, 0, 0, 0, 0, 0, 0x40, 0x20, 0x60 };

/*
        main monitor loop for device conditions
*/
void
dv_adv(dv)
        DV_             *dv;
{
        dv_cbk(dv);                       /* check for ending a break */
}

/*
        control the sending of a break
*/
void
dv_brk(bk, dv)
        int     bk;                     /* send break if TRUE */
        DV_             *dv;
{
        dv->dv_brks = (bk & 1) << 4;    /* save in the device table */
        WR5(dv_mr5(dv));                /* construct WR5 and send it */
        if (dv->dv_brks)                /* if just started a break */
                dv->dv_tbks = elapsed_ticks; /* set its time */
}


/*
        check whether a break has been sent long enough
*/
void
dv_cbk(dv)
        DV_             *dv;
{
        if (!dv->dv_brks)               /* is the line sending break */
                return;                     /* no, return */

        if ((elapsed_ticks - dv->dv_tbks) < 3) /* send break for 0.3 seconds */
                return;

        dv_brk(FALSE, dv);              /* turn off break */
}



/*
        control DTR modem control signal
*/
void
dv_dtr(b, dv)
        int         b;                  /* new value for DTR signal (0/1) */
        DV_             *dv;
{
        dv->dv_dtr = (b & 1) << 7;      /* save value in dtr byte */
        WR5(dv_mr5(dv));                /* construct WR5 and send it */
}


/*
        general initialization procedure for a configured port
*/
int
dv_ini(dv)
        DV_             *dv;
{   LC_             *lc;            /* corresponding line control table */
        unsigned int    tc;             /* time constant */
        int             po;             /* line number (0 - 15) */
        int             s;              /* flags state save */
        int             i;              /* work variable */
        char            cvs[10];        /* conversion string */
        char            outs[20];       /* output string */

        lc = dv->dv_lcad;               /* set up line table pointer */
        po = lc->lc_lnno;               /* get port number */
        dv->dv_cmdp = base_port[po];    /* command/status port */
        dv->dv_datp = base_port[po] + 2; /* data port */

        if ((dv->dv_bits + (dv->dv_par == 0 ? 0 : 1)) < 8)
                return(ER_TX_CHAR_SIZE);    /* notify host */

        if (dv->dv_spd != 0xFFFF)       /* normal specification */
                tc = intdiv(CLOCK_RATE, dv->dv_spd) - 2;
        else                            /* special flag for 9600 Bps */
                tc = (CLOCK_RATE / 96000L) - 2; /* set with constant value */
        s = dint();                     /* disable interrupts */
        WR0(0);                         /* insure correct polarity */
        WR9(po & 1 ? 0x40 : 0x80);      /* reset channel */
        WR14(0);                        /* clear reg 14 before setting time */
        WR4(dv->dv_par | dv->dv_stpb | 0x40); /* parity, stop bits, x16 clock */
        WR2(PORT_INT + ((po & ~1) << 3)); /* set base interrupt for VIS */
        i = bpc[dv->dv_bits] << 1;      /* add bits per character spec */
        WR3(i);                         /* configure receive port, no enable */
        i = bpc[dv->dv_bits];                   /* bits per character selector */
        WR5(i);                         /* configure transmit port, no enable */
        WR9(1);                         /* select VIS, status low */
        WR11(0x52);                     /* all clocks BR generator output */
        WR12(tc & 0xFF);                /* set BR constant (low) */
        WR13((tc >> 8) & 0xFF);         /* set BR constant (high) */
        WR14(0x83 + (dv->dv_loop ? LOOPBACK : 0));
                                                                        /* BR gen enable + source = BR gen +
                                                                           local loopback, if so configured   */
        WR0(0x28);                      /* reset Tx interrupt pending */
        WR15(0xA8);                     /* ext/status int controls */
                                                                        /* Break/abort, CTS and DCD IE */
        WR0(0x10);                      /* reset Ext/status interrupts */
        WR0(0x10);                      /* reset Ext/status interrupts (again) */
        WR1(0x17);                      /* Int enable: Tx, all Rx or special
                                                                           conditions, parity is special
                                                                           condition, External status change */
        WR9(9);                         /* VIS + MIE */
        WR0(0x30);                      /* reset error */
        WR0(0x38);                      /* reset highest IUS */
        dv_gr0(dv);                             /* get current RR0 status */
        eint(s);                        /* restore interrupt system */

        return(OK);                     /* return no errors */
}


/*
        control RTS bit
*/
void
dv_rts(b, dv)
        int         b;                  /* value to set RTS bit to (0/1) */
        DV_             *dv;
{
        dv->dv_rts = (b & 1) << 1;      /* save value in RTS byte */
        WR5(dv_mr5(dv));                /* construct WR5 and send it */
}


/*
        disable receiver
*/
void
dv_rxd(dv)
        DV_             *dv;
{
        WR3(0);                         /* output disable receiver */
}


/*
        enable receiver
*/
void
dv_rxe(dv)
        DV_             *dv;
{
        WR3((bpc[dv->dv_bits] << 1) + RXE); /* write out receive activator */
}


/*
        disable transmitter
*/
void
dv_txd(dv)
        DV_             *dv;
{
        dv->dv_txe = 0;                 /* set Tx enable off */
        WR5(dv_mr5(dv));                /* send the command register */
}


/*
        enable transmitter
*/
void
dv_txe(dv)
        DV_             *dv;
{
        dv->dv_txe = TXE;               /* set Tx enable on */
        WR5(dv_mr5(dv));                /* send the command register */
}


/*
        transmit next character
*/
void
dv_txn(dv)
        DV_             *dv;
{   LC_             *lc;

        lc = dv->dv_lcad;               /* get line control table */
        while (((dv_gr0(dv) & TBE) != 0) && (lc->lc_txcc > 0))
                                                                        /* as long as Tx buffer is empty */
        {
                WRCH(lc_gnc(lc));           /* write next Tx character */
        }
        if (lc->lc_txcc <= 0)           /* no more characters to send */
        {
                lc_ste(TD, lc);             /* signal line ctl last char sent */
        }
}


/*
        start transmitter
*/
void
dv_txs(dv)
        DV_             *dv;
{
        dv_txe(dv);                     /* enable transmitter */
        dv_txn(dv);                     /* put first character in transmitter */
}


/*
        receive buffer full interrupt service routine
*/
int
rxf_isr(po)
        int             po;             /* port which got the interrupt */
{   DV_             *dv;            /* device control block */
        LC_             *lc;            /* line control block */
        char            c;              /* character storage */

        dv = dvad[po];                  /* develop device ctl block addr */
        lc = dv->dv_lcad;               /* find corresponding line control addr */

        if (lc->lc_rxst == CLOSED)      /* line is closed: spurious int */
        {   RDCH();                     /* read and discard character */
                dv_rxd(dv);                 /* disable receiver */
                return(dv->dv_cmdp);
        }

        if (dv->dv_lrr0 & BRK_ABORT)    /* if line was in break */
        {   RDCH();                     /* toss NULL character */
                return(dv->dv_cmdp);        /* return */
        }

        while (dv_gr0(dv) & RXA)        /* while receive buffer is full */
        {   if ((c = RDCH() & 0x7F) != 0) /* read character, mask parity */
                        lc_rxn(c, lc);          /* process non-0 received character */
        }
        return(dv->dv_cmdp);            /* return port number for IUS reset */
}


/*
        transmit buffer empty interrupt service routine
*/
int
tbe_isr(po)
        int             po;             /* port which got the interrupt */
{   DV_             *dv;            /* device control block */
        LC_             *lc;            /* line control block */

        dv = dvad[po];                  /* develop device ctl block addr */
        lc = dv->dv_lcad;               /* find corresponding line control addr */

        if (lc->lc_txst == ACTIVE)      /* line is sending */
                dv_txn(dv);                     /* send next character[s] */
        else if (lc->lc_txst == CLOSED) /* line is closed: spurious int */
                dv_txd(dv);                 /* disable transmitter */

        WR0(0x28);                      /* clear Tx interrupt pending */
        return(dv->dv_cmdp);            /* return port number */
}


/*
        special receive condition interrupt service routine
*/
int
src_isr(po)
        int             po;             /* port which got the interrupt */
{   DV_             *dv;            /* device control block */
        LC_             *lc;            /* line control block */
        int             ss;             /* special receive condition status */

        dv = dvad[po];                  /* develop device ctl block addr */
        lc = dv->dv_lcad;               /* find corresponding line control addr */

        if (lc->lc_rxst == CLOSED)      /* receiver is closed: spurious int */
        {   dv_rxd(dv);                 /* disable receiver */
                WR0(0x30);                  /* reset errors */
                return(dv->dv_cmdp);
        }

        ss = RR(1);                     /* get status */

        if (ss & FRAMING_ERROR)         /* if framing error occurred */
                lc->lc_errs |= ME_FRAMING;  /* note framing error in msg */

        if (ss & PARITY_ERROR)          /* if parity error occurred */
                lc->lc_errs |= ME_PARITY;   /* note parity error in msg */

        if (ss & RX_OVERRUN)            /* Rx overrun occurred */
                lc->lc_errs |= ME_OVERRUN;  /* note Rx overrun error */

        WR0(0x30);                      /* reset errors */
        return(dv->dv_cmdp);            /* return port number */
}


/*
        external status change interrupt service routine
*/
int
esc_isr(po)
        int             po;             /* port which got the interrupt */
{   DV_             *dv;            /* device control block */
        LC_             *lc;            /* line control block */
        int             r;              /* temporary hold */
        int             diff;           /* statuses changed since last time */

        dv = dvad[po];                  /* develop device ctl block addr */
        lc = dv->dv_lcad;               /* find corresponding line control addr */

        if ((lc->lc_rxst == CLOSED) && (lc->lc_txst == CLOSED))
                                                                        /* line is shut down: spurious int */
        {   dv_txd(dv);                 /* disable transmitter */
                dv_rxd(dv);                 /* disable receiver */
                WR0(0x10);                  /* reset EXT/STATUS interrupts */
                return(dv->dv_cmdp);
        }

        r = RR(0);                      /* status register */
        diff = (r ^ dv->dv_lrr0) & (DCD | CTS | BRK_ABORT); /* what's changed */

        dv->dv_lrr0 = r;

        if (diff & DCD)                 /* if DCD changed */
                lc_sre(DC, lc);                         /* signal line control of change */

        if (diff & CTS)                 /* if CTS changed */
                lc_sre(CT, lc);                         /* signal line control of change */

        if (diff & BRK_ABORT)           /* if BREAK/ABORT changed */
        {   if (r & BRK_ABORT)          /* if break begins */
                {   dv->dv_rbks = elapsed_ticks; /* record time of break start */
                        lc_sre(BK, lc);                 /* signal line control of break start */
                }
                else
                {       lc_sre(BE, lc);                 /* signal line control of break end */
                }
        }

        WR0(0x10);                      /* reset EXT/STATUS interrupts */
        return(dv->dv_cmdp);            /* return port number */
}


                                        /* ***** END OF FILE ***** */
