/**********************************************************************
*  File:        HO.C                                                  *
*  Purpose:     COMTROL Hostess i Programmer's Toolkit Host           *
*               Communications Functions                              *
*  Date:        September 20, 1990                                    *
*  Author:      Howard D. Straus                                      *
*  Release:     1.00 - Original release                               *
***********************************************************************
*                         COPYRIGHT NOTICE                            *
*=====================================================================*
*   Copyright 1990, 1991 by Howard D. Straus and Comtrol Corporation. *
*   All rights reserved.  Use or reproduction by any means of any     *
*   portion of this code contrary to the developers license agreement *
*   is expressly prohibited.                                          *
***********************************************************************/

#include "compiler.h"

#include <stdlib.h>

#include "hostess.h"
#include "proto.h"
#include "externs.h"

#define norm(x)  (((long)FP_SEG(x) << 4) + (long)FP_OFF(x))

/*
        monitor host check entry
*/
void
ho_aho()
{
        ho_cho();                   /* check host output */
        ho_chi();                   /* check host input */
}


/*
        queue an alarm message to the host
*/
int
ho_alr(er, lc)
        int             er;             /* error number */
        LC_            *lc;             /* LCT address */
{   BF_     far    *bp;             /* buffer pointer */

        if ((bp = bq_alc()) == NULL)    /* no buffers, ignore */
                return(ER);             /* return "not sent" */

        bp->bf_type = ALARM;            /* indicate line alarm */
        bp->bf_text[0] = er;            /* put error code in buffer */
        bp->bf_txct = 1;                /* one byte of error information */
        bp->bf_lnno = lc->lc_lnno;      /* line number */
        bp->bf_tmst = curr_time;        /* timestamp buffer */
        bq_enq(bp, (QC_ far *)&host_queue); /* queue message to the host */
        return(OK);                     /* return no error */
}


/*
        check if host has data for Hostess i
*/
void
ho_chi()
{   MG_     far    *mg;             /* message buffer pointer */
        TIME    far    *tp;             /* time pointer */
        int             ln;             /* line number */
        char            ts[40];         /* for time string */

        if (mb->mb_hoct == 0)           /* no message in buffer */
                return;

        mg = mb->mb_homg;               /* get address of host message buffer */

        switch (mg->mg_type) {          /* what kind of command */

        case WRITE_DATA:                /* data to send to line */
                curr_line = mg->mg_line;    /* save the current line */
                if (ho_wrd(mg))             /* queue data to the line */
                        return;                 /* do not clear host buffer */
                break;

        case CLOSE_LINE:                /* close the line */
                curr_line = mg->mg_line;    /* save the current line */
                ho_lnc(mg);
                break;

        case OPEN_LINE:                 /* open the line */
                curr_line = mg->mg_line;    /* save the current line */
                ho_lno(mg);
                break;

        case CONFIG_PORT:               /* config port characteristics */
                curr_line = mg->mg_line;    /* save the current line */
                if (ho_dcf(mg))             /* configure the port */
                        return;                 /* if error, do not clear buffer */
                break;

        case MODEM_CONTROL:             /* set/clear modem control signals */
                curr_line = mg->mg_line;    /* save the current line */
                ho_mcs(mg);                 /* do the trick */
                break;

        case ENTER_DEBUGGER:            /* enter the Hostess i debugger */
                debug();                    /* jump to ROM debugger */
                break;                      /* return when 'G' is typed */

        case SEND_BREAK:                /* send 0.3 second break */
                curr_line = mg->mg_line;    /* save the current line */
                dv_brk(TRUE, dvad[curr_line]); /* command is immediate */
                break;

        case SET_TIME:                  /* set the time of day */
                ho_stm(mg);                 /* set the Hostess i clock */
                break;

        default:                        /* unknown command */
                if (ho_err(ER_UNKNOWN))     /* error message */
                        return;                 /* do not clear buffer */
                break;
        }

        mb->mb_hoct = 0;                /* indicate data used */
}


/*
        check if anything is to be sent to the host
*/
void
ho_cho()
{   BF_     far    *bp;             /* buffer pointer */
        MG_     far    *mg;             /* Hostess i message area */
        RB_     far    *rb;             /* received data message pointer */
        UTINY   far    *dp;             /* copy character pointer */
        int             i;              /* loop variable */
        int             lh;             /* length of data */
        int             space;          /* space remaining in host buffer */

        if (bq_qem((QC_ far *)&host_queue)) /* if nothing to send */
                return;

        if (mb->mb_ulct != 0)           /* if host buffer not clear */
                return;                     /* do not disturb */

        if ((bp = bq_deq((QC_ far *)&host_queue)) == NULL)
                return;                         /* no buffer found */

        mg = mb->mb_ulmg;               /* get address of Hostess i buffer */
        fmemset((char far *)mg, 0, HOST_MSG_SIZE); /* clear buffer to 0 */

        switch (mg->mg_type = bp->bf_type) { /* switch on message type */

        case STATUS_REPORT:             /* line status report */
                time_pending = FALSE;           /* note time sent */
        case ERROR:                     /* host message error */
        case ALARM:                     /* line failure alarm */
        case LINE_ERRORS:               /* line error report */
                mg->mg_line = bp->bf_lnno;      /* copy line number */
                ffmemcpy((char far *)mg->mg_data, (char far *)bp->bf_text,
                         bp->bf_txct);          /* copy entire buffer */
                mb->mb_ulct = bp->bf_txct + MG_HDR; /* set data available */
                ho_int();                   /* interrupt host */
                break;

        case DATA_BUFFER:               /* message received from line */
                ffmemcpy((char far *)mg, (char far *)&bp->bf_type,
                        bp->bf_txct + RB_HDR);  /* move buffer to message area */
                mb->mb_ulct = bp->bf_txct + RB_HDR;
                                                                        /* add flags, timestamp, MG_ hdr */
                ho_int();                   /* interrupt host */
                break;

        default:
                break;
        }

        bq_fre(bp);                     /* release the buffer */
}


/*
        configure a device
*/
int
ho_dcf(mg)
        MG_     far    *mg;             /* host message */
{   LC_            *lc;             /* LCT address */
        DV_            *dv;             /* DCT address */
        int             er;             /* error return */

        dv = dvad[mg->mg_line];         /* get DCT address */
        lc = dv->dv_lcad;               /* and LCT address */

        if (mb->mb_hoct <= 2)           /* no configuration data in buffer */
                return(OK);                 /* ignore call */

        if (lc->lc_txst != CLOSED)              /* if transmitter open */
                lc_clt(lc);                 /* close transmitter */

        if (lc->lc_rxst != CLOSED)              /* if receiver open */
                lc_clr(lc);                 /* close receiver */

        ffmemcpy((char far *)&dv->dv_spd, mg->mg_data,
                                        min(mb->mb_hoct - 2, sizeof(CD_)));
                                                                /* copy parameters to DCT */
        /*
                consistency checks
        */
        if ((dv->dv_bits != 7) && (dv->dv_bits != 8))
                                                                        /* impermissible character size */
        {   dv->dv_spd = 0;             /* clear "configured" flag */
                return(ho_err(ER_RX_CHAR_SIZE));
        }
        if ((dv->dv_bits == 7) && (dv->dv_par == 0)) /* not enough bits */
        {   dv->dv_spd = 0; /* clear "configured" flag */
                return(ho_err(ER_PARITY_REQ));
        }

        if ((er = dv_ini(dv)) != 0)     /* if errors in setting up the chip */
                return(ho_err(er));         /* tell the host */

        return(OK);                     /* configuration accepted */
}

/*
        send host a diagnostic text message
*/
int
ho_dig(s, ln)
        char           *s;              /* diagnostic text string */
        int             ln;             /* line number to display */
{   BF_     far    *bp;             /* work buffer pointer */
        char            ls[5];          /* string for line number conversion */

        if ((bp = bq_alc()) == NULL)    /* no buffer available */
                return(-1);                                     /* no diagnostic sent */

        fstrcpy(bp->bf_text, "// Line "); /* prepend diagnostic header */
        fstrcat(bp->bf_text, itoa(ln + 1, ls, 10)); /* add line number */
        fstrcat(bp->bf_text, " : ");    /* separate from text */
        fstrcat(bp->bf_text, s);        /* add diagnostic text */
        bp->bf_lnno = ln;               /* put line number in buffer */
        bp->bf_txct = fstrlen(bp->bf_text); /* text length to buffer */
        bp->bf_tmst = curr_time;        /* timestamp buffer */
        bp->bf_type = DATA_BUFFER;      /* designate as a data buffer */
        bq_enq(bp, (QC_ far *)&host_queue); /* add to output queue */
        return(0);                      /* return, diagnostic sent */
}

/*
        queue an error message to the host
*/
int
ho_err(er)
        int             er;             /* error number */
{   BF_     far     *bp;            /* buffer pointer */

        if ((bp = bq_alc()) == NULL)    /* no buffers */
                return(ER);             /* return "did not send" */

        bp->bf_type = ERROR;            /* indicate host error */
        bp->bf_text[0] = er;            /* put error code in buffer */
        bp->bf_txct = 1;                /* one byte of error information */
        bp->bf_tmst = curr_time;        /* timestamp buffer */
        bq_enq(bp, (QC_ far *)&host_queue); /* queue message to the host */
        return(OK);                     /* no error */
}


/*
        interrupt the host processor
*/
void
ho_int()
{   int         i;

        OUTP(HOST_INTERRUPT_PORT, SET_INTERRUPT); /* set the host IRQ interrupt */

        for (i = 0; i < 3; i++)                         /* delay a few ticks */
                ;

        OUTP(HOST_INTERRUPT_PORT, CLEAR_INTERRUPT); /* clear the host interrupt */
}


/*
        service a host generated INT1
*/
void
ho_isr()
{
        host_interrupt_flag++;              /* notify cp of host message */
}


/*
        close a line
*/
void
ho_lnc(mg)
        MG_     far     *mg;            /* host message */
{   LC_             *lc;            /* LCT address */
        int             i;              /* work */

        lc = dvad[mg->mg_line]->dv_lcad; /* develop LCT address */

        i = mg->mg_data[0];             /* get the I/O indicator */

        if ((i & 1) != 0)               /* if close input request */
                lc_sre(CL, lc);             /* signal line control */

        if ((i & 2) != 0)               /* if close output request */
                lc_ste(CL, lc);             /* signal line control */
}


/*
        open a line
*/
int
ho_lno(mg)
        MG_     far     *mg;            /* host message */
{   LC_             *lc;            /* LCT address */
        int             i;              /* work */

        lc = dvad[mg->mg_line]->dv_lcad; /* develop LCT address */
        i = mg->mg_data[0];             /* get the I/O indicator */

        if ((i & 1) != 0)               /* request open input */
        {   if (lc->lc_rxst != CLOSED)
                        return(ho_err(ER_OPEN_ACTIVE_LINE));
                lc_sre(OP, lc);             /* signal line control */
        }

        if ((i & 2) != 0)               /* request open output */
        {   if (lc->lc_txst != CLOSED)  /* line is not closed */
                        return(ho_err(ER_OPEN_ACTIVE_LINE));
                lc_ste(OP, lc);             /* signal line control */
        }

        return(OK);                     /* no errors */
}


/*
        set the states of the controllable modem control signals
*/
void
ho_mcs(mg)
        MG_     far     *mg;            /* message buffer pointer */
{
        dv_rts(mg->mg_data[0] >> 1, dvad[mg->mg_line]);
                                                                        /* set the RTS signal level */
        dv_dtr(mg->mg_data[0] >> 7, dvad[mg->mg_line]);
                                                                        /* set the DTR signal level */
        ho_sls(dvad[mg->mg_line]->dv_lcad); /* reflect status to host */
}


/*
        queue a line error message to the host
*/
int
ho_sle(er, lc)
        int             er;             /* error number */
        LC_             *lc;            /* line which had the error */
{   BF_     far     *bp;            /* buffer pointer */

        if ((bp = bq_alc()) == NULL)    /* no buffers */
                return(ER);             /* return "did not send" */

        bp->bf_type = LINE_ERRORS;      /* indicate line error */
        bp->bf_txct = 1;                /* one byte of error information */
        bp->bf_text[0] = er;            /* put error code in buffer */
        bp->bf_lnno = lc->lc_lnno;      /* line number */
        bp->bf_tmst = curr_time;        /* timestamp buffer */
        bq_enq(bp, (QC_ far *)&host_queue); /* queue message to the host */
        return(OK);                     /* no error */
}


/*
        send the line status to the host
*/
int
ho_sls(lc)
        LC_             *lc;            /* LCT address */
{   BF_     far     *bp;            /* buffer pointer */
        LS_     far     *ls;            /* line status structure */
        DV_             *dv;            /* DCT address */

        if ((bp = bq_alc()) == NULL)    /* if no buffer available */
                return(ER);                 /* return, not sent */

        dv = lc->lc_dvad;               /* get corresponding DCT */
        ls = (LS_ far *)bp->bf_text;    /* point to line status structure */
        ffmemcpy((char far *)&ls->ls_time, (char far *)&curr_time, sizeof(TIME));
        ffmemcpy((char far *)&ls->ls_dctb, (char far *)dv, sizeof(DV_));
        ffmemcpy((char far *)&ls->ls_lctb, (char far *)lc, sizeof(LC_));
        bp->bf_txct = sizeof(LC_) + sizeof(DV_) + sizeof(TIME);
        bp->bf_tmst = curr_time;        /* timestamp buffer */
        bp->bf_type = STATUS_REPORT;    /* buffer type */
        bp->bf_lnno = lc->lc_lnno;      /* line number */
        bq_enq(bp, &host_queue);        /* enqueue the message to the host */

        time_pending = TRUE;                    /* note time in queue */

        return(OK);                     /* completed request */
}


/*
        set the Hostess i time
*/
void
ho_stm(mg)
        MG_     far     *mg;            /* host message buffer */
{   TIME    far     *tp;            /* time pointer */
        int             s;              /* save flags register */

        tp = (TIME far *)mg->mg_data;   /* point to new time of day */
        s = dint();                     /* disable interrupts */
        curr_time = *tp;                /* move to curr time */
        eint(s);                        /* enable interrupts */
        ho_sls(&lct[curr_line]);        /* send response */
}


/*
        add write data to an output line transmit queue
*/
int                                 /* return non-zero if unable to use */
ho_wrd(mg)
        MG_     far     *mg;            /* host message pointer */
{   BF_     far     *bp;            /* buffer pointer */
        BF_     far     *bf;            /* buffer pointer */
        LC_             *lc;            /* LCT address */
        char    far     *dp;            /* work pointer */
        int             flags;          /* hold */
        int             dtct;           /* data count */

        lc = dvad[mg->mg_line]->dv_lcad; /* get LCT address */

        if (lc->lc_txst == CLOSED)      /* if line is closed */
        {   ho_err(ER_WRITE_TO_CLOSED_LINE); /* send error message */
                return(OK);                 /* toss data */
        }

        if (n_free_buffers < STOP_HOST_RX_THRESH) /* not enough buffers */
                return(ER);                 /* try again another time */

        if ((bp = bq_alc()) == NULL)    /* get a buffer */
                return(ER);                 /* none available, try later */

        /*
                now append message characters to line output queue
        */
        dp = mg->mg_data;               /* point to first data character */
        if ((dtct = mb->mb_hoct - MG_HDR) > 0) /* number of text characters */
        {   bp->bf_txct = dtct;          /* put into buffer */
                ffmemcpy(bp->bf_text, dp, dtct); /* copy text to buffer */
                bq_enq(bp, &lc->lc_txqc);   /* add to output queue */
        }
        return(OK);                     /* return data used */
}



                                        /* ***** END OF FILE ***** */

