/************************************************************************
*  File:     PROTO.H                                                    *
*  Purpose:  Function Prototypes for each Hostess i Board function      *
*  Author:   Howard D. Straus                                           *
*  Date:     February 15, 1991                                          *
*  Release:  1.00 - Original release                                    *
*************************************************************************
*                         COPYRIGHT NOTICE                            *
*=====================================================================*
*   Copyright 1990, 1991 by Howard D. Straus and Comtrol Corporation. *
*   All rights reserved.  Use or reproduction by any means of any     *
*   portion of this code contrary to the developers license agreement *
*   is expressly prohibited.                                          *
***********************************************************************/

#define MIN(x, y)       (x < y ? x : y)
#define MAX(x, y)       (x > y ? x : y)

/****************************************************/
/*     Assembly Language Functions (C0S and CRT0)   */
/****************************************************/

void            debug(void);         /* enter the on-board debugger */
long            intdiv(long, int);   /* integer divide */
int             intmod(long, int);   /* integer modulo */


/******************************************/
/*      Buffer Queue Manager (BQ.C)       */
/******************************************/

BF_     far     *bq_alc(void);       /* allocate a buffer */
BF_     far     *bq_deq(QC_ far *);  /* dequeue a buffer */
void             bq_enq(BF_ far  *, QC_ far *); /* enqueue a buffer */
void             bq_fpq(QC_ far *);  /* free all buffers on a queue */
void             bq_fre(BF_ far  *); /* free a buffer */
void             bq_ini(void);       /* initialize buffer pool */
void             bq_qci(QC_ far *);  /* initialize a queue control block */
int              bq_qem(QC_ far *);  /* check if queue is wmpty */


/********************************************/
/*      Control Program Module (CP.C)       */
/********************************************/

void             cp_acp(void);       /* main Hostess i control program */
void             cp_ini(void);       /* initialize system */


/*******************************************/
/*      Device Control Module (DV.C)       */
/*******************************************/

void             dv_adv(DV_ *);      /* main device control entry */
void             dv_brk(int, DV_ *); /* set or clear break condition */
void             dv_cbk(DV_ *);      /* check for break conditions */
void             dv_dtr(int, DV_ *); /* set condition of DTR pin */
int              dv_ini(DV_ *);      /* initialize port */
void             dv_rts(int, DV_ *); /* control RTS */
void             dv_rxd(DV_ *);      /* disable receiver */
void             dv_rxe(DV_ *);      /* enable receiver */
void             dv_txd(DV_ *);      /* disable transmitter */
void             dv_txe(DV_ *);      /* enable transmitter */
void             dv_txn(DV_ *);      /* transmit next character */
void             dv_txs(DV_ *);      /* start transmission on port */
int              rxf_isr(int);       /* receiver full interrupt service */
int              tbe_isr(int);       /* transmit buffer empty int service */
int              esc_isr(int);       /* external status interrupt service */
int              src_isr(int);       /* spec rx condition int service */


/**********************************************/
/*     Far String/Memory Functions (FSTR.C)   */
/**********************************************/

void    far     *fmemset(char far *, int, int); /* set memory */
void    far     *ffmemcpy(char far *, char far *, int); /* copy memory */

char    far     *fstrcat(char far *, char *); /* concatenate strings */
char    far     *fstrcpy(char far *, char *); /* copy strings */
char    far     *ffstrcpy(char far *, char far *); /* copy strings */
int              fstrlen(char far *);   /* get string length */
int              ffstrncmp(char far *, char far *, int); /* compare strings */
char    far     *ffstrstr(char far *, char far *);
                                                                                /* find one string in another */


/*******************************************/
/*      Host Interface Module (HO.C)       */
/*******************************************/

void             ho_aho(void);       /* main host entry */
int              ho_alr(int, LC_ *); /* send host alarm message */
void             ho_chi(void);       /* check host input */
void             ho_cho(void);       /* check host output */
int              ho_dcf(MG_ far *);  /* configure a device */
int              ho_err(int);        /* host, queue error message */
void             ho_int(void);       /* interrupt the host processor */
void             ho_isr(void);       /* ISR for host interrupt */
void             ho_lnc(MG_ far *);  /* close a line */
int              ho_lno(MG_ far *);  /* open a line */
void             ho_mcs(MG_ far *);  /* set modem control signals */
int              ho_sle(int, LC_ *); /* send line error message */
int              ho_sls(LC_ *);      /* send line status msg */
void             ho_stm(MG_ far *);  /* set Hostess i date/time */
int              ho_wrd(MG_ far *);  /* queue data to a port */


/*****************************************/
/*      Line Control Module (LC.C)       */
/*****************************************/

void             lc_alc(LC_ *);      /* main line control dispatcher */
int              lc_clr(LC_ *);      /* close receiver */
int              lc_clt(LC_ *);      /* close transmitter */
int              lc_gnc(LC_ *);      /* get next char to transmit */
int              lc_har(LC_ *);      /* handle active receiver */
int              lc_hat(LC_ *);      /* handle active transmitter */
int              lc_hcr(LC_ *);      /* handle closed receiver */
int              lc_hct(LC_ *);      /* handle closed transmitter */
int              lc_hit(LC_ *);      /* handle idle transmitter */
int              lc_hlf(LC_ *);      /* handle line failure */
int              lc_rxf(LC_ *);      /* finish receive */
void             lc_rxn(int, LC_ *); /* receive next character */
int              lc_rxs(LC_ *);      /* start receive */
void             lc_sch(int, LC_ *); /* store next character */
void             lc_sre(int, LC_ *); /* send receive event */
void             lc_ste(int, LC_ *); /* send transmit event */
int              lc_txf(LC_ *);      /* finish transmit */
int              lc_txs(LC_ *);      /* start transmit */
int              lc_wbe(LC_ *);      /* wait for break to end */
int              lc_wfr(LC_ *);      /* wait for receive event */
int              lc_wft(LC_ *);      /* wait for transmit event */


/*******************************************/
/*      Memory Management Module (MEM.C)   */
/*******************************************/

void     far    *memalloc(unsigned int); /* allocate page 0 memory */
unsigned int     memavail(void);     /* amount of memory available */


/*****************************************/
/*      Timer Module Functions (TM.C)    */
/*****************************************/

void             tm_ini(void);       /* initialize timer functions */
void             tmr_isr(void);      /* timer interrupt service routine */


                                        /* ***** END OF FILE ***** */


